/* ss-core/main.c
 *
 * Entry point for the SmartShelf core program.
 *
 * Copyright (C) 2003, 2004
 * Andy Goth     <unununium@openverse.com>
 *
 * This code is available under the GNU General Public License; see COPYING. */

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/poll.h>
#include <unistd.h>

#include "evlist/evlist.h"

extern char** environ;

static evlist_t el;
static device_t swap, tty;

static void sig_cleanup  (int sig);
static int  swap_init    (device_t* dev);
static int  swap_incoming(device_t* dev);
static int  swap_send    (device_t* dev, char* text, int len);
static int  tty_init     (device_t* dev);
static int  tty_close    (device_t* dev);
static int  tty_incoming (device_t* dev);
static int  tty_send     (device_t* dev, char* text, int len);

static void sig_cleanup(int sig)
{
    signal(sig, SIG_DFL);
    evlist_loop_end(&el);
}

static int swap_init(device_t* dev)
{
    dev->sw_incoming = swap_incoming;
    return 0;
}

static int swap_incoming(device_t* dev)
{
    tty_send(&tty, VEC_ELTS(byte_t, &dev->buf_in),
                   VEC_LEN(byte_t, &dev->buf_in));
    VEC_TRUNC(byte_t, &dev->buf_in, 0);
    return 0;
}

static int swap_send(device_t* dev, char* text, int len)
{
    VEC_PUSHN(byte_t, &dev->buf_out, text, len);
    dev->pollfd_out->events |= POLLOUT;
    return 0;
}

static int tty_init(device_t* dev)
{
    dev->sw_close    = tty_close   ;
    dev->sw_incoming = tty_incoming;
    return 0;
}

static int tty_close(device_t* dev)
{
    evlist_loop_end(dev->evlist);
}

static int tty_incoming(device_t* dev)
{
    swap_send(&swap, VEC_ELTS(byte_t, &dev->buf_in),
                     VEC_LEN(byte_t, &dev->buf_in));
    VEC_TRUNC(byte_t, &dev->buf_in, 0);
    return 0;
}

static int tty_send(device_t* dev, char* text, int len)
{
    VEC_PUSHN(byte_t, &dev->buf_out, text, len);
    dev->pollfd_out->events |= POLLOUT;
    return 0;
}

int main(int argc, char** argv)
{
    evlist_init(&el);

    device_init(&swap, "swap case");
    child_init(&swap, "misc/swapcase", environ, O_RDWR);
    /* child_init(&swap, "tr a-zA-Z A-Za-z", environ, O_RDWR); */
    /* child_init(&swap, "misc/rnd_pres | xxd -p -c 80", environ, O_RDWR); */
    swap_init(&swap);
    evlist_device_add(&el, &swap);

    device_init(&tty, "tty");
    fd_init(&tty, "tty", STDIN_FILENO, STDOUT_FILENO);
    tty_init(&tty);
    evlist_device_add(&el, &tty);

    signal(SIGINT, sig_cleanup); signal(SIGTERM, sig_cleanup);
    evlist_loop(&el);
    signal(SIGINT, SIG_DFL); signal(SIGTERM, SIG_DFL);

    evlist_cleanup(&el);

    device_cleanup(&swap);
    device_cleanup(&tty);

    return EXIT_SUCCESS;
}

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
