/* ss-hw/main.c
 *
 * Entry point of the Smartshelf hardware interface program.
 *
 * Copyright (C) 2003, 2004
 * Andy Goth     <unununium@openverse.com>
 *
 * This code is available under the GNU General Public License; see COPYING. */

#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/poll.h>
#include <termios.h>
#include <unistd.h>

#include "evlist/evlist.h"
#include "ss-hw/pressure.h"
#include "ss-hw/rfid.h"

static evlist_t el;

static void sig_cleanup(int sig)
{
    signal(sig, SIG_DFL);
    evlist_loop_end(&el);
}

int main(int argc, char** argv)
{
    device_t pressure, rfid;

    /* Because I was having trouble with this earlier... */
    setvbuf(stdout, NULL, _IONBF, 0);

    evlist_init(&el);

    device_init(&pressure, "pressure sensor");
    fd_init(&pressure, "stdin", STDIN_FILENO, STDOUT_FILENO);
    /* serial_init(&pressure, "/dev/ttyS1", CS8, 0, 0, B9600, 0, 0); */
    pressure_init(&pressure);
    evlist_device_add(&el, &pressure);

    device_init(&rfid, "RFID tag reader");
    /* fd_init(&rfid, "stdin", STDIN_FILENO, STDOUT_FILENO); */
    serial_init(&rfid, "/dev/ttyS1", CS8, 0, PARENB, B38400, 0, 0);
    rfid_init(&rfid);
    evlist_device_add(&el, &rfid);

    signal(SIGINT, sig_cleanup); signal(SIGTERM, sig_cleanup);
    evlist_loop(&el);
    signal(SIGINT, SIG_DFL); signal(SIGTERM, SIG_DFL);

    evlist_cleanup(&el);

    device_cleanup(&pressure);
    device_cleanup(&rfid);

    return EXIT_SUCCESS;
}

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
